/* This file is an image processing operation for GEGL
 *
 * GEGL is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * GEGL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
 *
 * Copyright 2006 Øyvind Kolås <pippin@gimp.org>
 * Glass over Text 2023 Beaver
 */

#include "config.h"
#include <glib/gi18n-lib.h>

#ifdef GEGL_PROPERTIES


/*
GEGL Graph Below
 */

/*
gaussian-blur std-dev-x=3 std-dev-y=3
emboss azimuth=44 depth=29 
id=1
gimp:layer-mode layer-mode=erase composite-mode=auto aux=[ ref=1 dst-over aux=[ color value=#000000  ]  crop  color-to-alpha opacity-threshold=0.14 color-overlay value=#000000 ]  ]
color-overlay value=#ffffff
 */



property_string (string, _("Graph"), TUTORIAL)
    ui_meta     ("role", "output-extent")

#define TUTORIAL \
" id=1 gimp:layer-mode layer-mode=erase composite-mode=auto aux=[ ref=1 dst-over aux=[ color value=#000000  ]  crop  color-to-alpha opacity-threshold=0.14 color-overlay value=#000000 ]  ]  "\





property_double (azimuth, _("Azimuth"), 30.0)
    description (_("Light angle (degrees)"))
    value_range (0, 360)
    ui_meta ("unit", "degree")
    ui_meta ("direction", "ccw")


property_int (depth, _("Depth"), 20)
    description (_("Filter width"))
    value_range (10, 100)


property_double (elevation, _("Elevation"), 45.0)
    description (_("Elevation angle (degrees)"))
    value_range (40, 46)
    ui_meta ("unit", "degree")


property_double (std_dev, _("Blur Shine"), 3.0)
   description (_("Add a Gaussian Blur"))
   value_range (1.5, 7)
   ui_range    (1.5, 7)
   ui_gamma    (3.0)
   ui_meta     ("unit", "pixel-distance")

property_color (color, _("Color"), "#ffffff")
    description (_("The color to paint over the input"))

property_double (hyperopacity, _("Hyper Opacity"), 1.0)
    description (_("Opacity Above 100 to make effect more noticable"))
    value_range (1.0, 1.5)
    ui_range    (1.0, 1.5)



#else

#define GEGL_OP_META
#define GEGL_OP_NAME     glassovertext
#define GEGL_OP_C_SOURCE glassovertext.c

#include "gegl-op.h"

static void attach (GeglOperation *operation)
{
  GeglNode *gegl = operation->node;
  GeglNode *input, *output, *emboss, *color, *gaussian, *hyperopacity, *string;

  input    = gegl_node_get_input_proxy (gegl, "input");
  output   = gegl_node_get_output_proxy (gegl, "output");


  emboss = gegl_node_new_child (gegl,
                                  "operation", "gegl:emboss",
                                  NULL);

  string = gegl_node_new_child (gegl,
                                  "operation", "gegl:gegl",
                                  NULL);

  hyperopacity = gegl_node_new_child (gegl,
                                  "operation", "gegl:opacity",
                                  NULL);




  gaussian    = gegl_node_new_child (gegl,
                                  "operation", "gegl:gaussian-blur",
                                  NULL);

  color    = gegl_node_new_child (gegl,
                                  "operation", "gegl:color-overlay",
                                  NULL);




  gegl_node_link_many (input, gaussian, emboss, string, color, hyperopacity, output, NULL);




  gegl_operation_meta_redirect (operation, "std_dev", gaussian, "std-dev-x");
  gegl_operation_meta_redirect (operation, "std_dev", gaussian, "std-dev-y");
  gegl_operation_meta_redirect (operation, "string", string, "string");
  gegl_operation_meta_redirect (operation, "azimuth", emboss, "azimuth");
  gegl_operation_meta_redirect (operation, "elevation", emboss, "elevation");
  gegl_operation_meta_redirect (operation, "depth", emboss, "depth");
  gegl_operation_meta_redirect (operation, "color", color, "value");
  gegl_operation_meta_redirect (operation, "hyperopacity", hyperopacity, "value");



}

static void
gegl_op_class_init (GeglOpClass *klass)
{
  GeglOperationClass *operation_class;

  operation_class = GEGL_OPERATION_CLASS (klass);

  operation_class->attach = attach;

  gegl_operation_class_set_keys (operation_class,
    "name",        "gegl:glassovertext",
    "title",       _("Glass Over Text"),
    "categories",  "Artistic",
    "reference-hash", "stglass238a5125700fc254001b2ac",
    "description", _("Glass Over Text Effect - Filter expects user to place effect on layer above or use the normal blend mode. It may even be a good idea to experiment with other blend modes."),
    NULL);
}

#endif
